import ctypes
import dataclasses
import subprocess
from ctypes import wintypes
from enum import Enum
from typing import Callable

import Xlib
import cv2
import numpy as np
from pynput import keyboard


MODIFIERS = {
    keyboard.Key.ctrl,
    keyboard.Key.ctrl_r,
    keyboard.Key.ctrl_l,
    keyboard.Key.alt,
    keyboard.Key.alt_r,
    keyboard.Key.alt_l,
    keyboard.Key.shift,
    keyboard.Key.shift_r,
    keyboard.Key.shift_l,
}


class SupportedOS(Enum):
    LINUX = "Linux"
    WINDOWS = "Windows"


@dataclasses.dataclass
class KeyFunction:
    key: str
    func: Callable
    description: str


def write_text_on_img(img, text, col=10, row=10, font_scale=1.5, color=(255, 255, 255), thickness=2, spacing=30):
    """
    Wrapper for cv2's putText with defaults and meaningful arg names
    """
    fontface = cv2.FONT_HERSHEY_PLAIN
    (_, height), baseline = cv2.getTextSize(text, fontface, font_scale, thickness)
    baseline += thickness

    cv2.putText(img, text, (col, row + height + baseline), fontface, font_scale, color, thickness)
    return row + spacing


def get_foreground_window_pid():
    h_wnd = ctypes.windll.user32.GetForegroundWindow()
    pid = ctypes.wintypes.DWORD()
    ctypes.windll.user32.GetWindowThreadProcessId(h_wnd, ctypes.byref(pid))
    return pid.value


def hist_eq_uint16(img):
    hist, bins = np.histogram(img.flatten(), 65536, (0, 65536))  # Collect 16 bits histogram (65536 = 2^16).
    cdf = hist.cumsum()

    cdf_m = np.ma.masked_equal(cdf, 0)  # Find the minimum histogram value (excluding 0)
    cdf_m = (cdf_m - cdf_m.min()) * 65535 / (cdf_m.max() - cdf_m.min())
    cdf = np.ma.filled(cdf_m, 0).astype("uint16")

    # Now we have the look-up table...
    hist_eq_img = cdf[img]
    return hist_eq_img


def get_screen_size_linux():
    try:
        screen_size_str = (
            subprocess.check_output('xrandr | grep "\*" | cut -d" " -f4', shell=True).decode().strip().split("\n")[0]
        )
        screen_w, screen_h = screen_size_str.split("x")
        screen_w = 0.9 * int(screen_w)
        screen_h = 0.9 * int(screen_h)
        return screen_w, screen_h
    except (ValueError, TypeError, subprocess.CalledProcessError, IndexError):
        return None, None


def get_screen_size_windows():
    user32 = ctypes.windll.user32
    screensize = 0.9 * user32.GetSystemMetrics(0), 0.9 * user32.GetSystemMetrics(1)
    return screensize


def get_screen_adjusted_frame_size(screen_size, frame_width, frame_height):
    screen_w, screen_h = screen_size
    if screen_w is None:
        return frame_width, frame_height

    adjusted_w = screen_w
    adjusted_h = screen_h
    if screen_w != frame_width:
        w_ratio = screen_w / frame_width
        adjusted_w = screen_w
        adjusted_h = frame_height * w_ratio
    if screen_h < adjusted_h:
        h_ratio = screen_h / adjusted_h
        adjusted_h = screen_h
        adjusted_w *= h_ratio

    return int(adjusted_w), int(adjusted_h)


def is_window_closed_by_mouse_click(window_name):
    return cv2.getWindowProperty(window_name, cv2.WND_PROP_VISIBLE) < 1


def get_in_focus_window_id(os: SupportedOS):
    if os == SupportedOS.LINUX:
        window = Xlib.display.Display().get_input_focus().focus
        if isinstance(window, int):
            return ""
        return window.get_wm_name()
    if os == SupportedOS.WINDOWS:
        return get_foreground_window_pid()
    else:
        raise NotImplementedError(f"{os=} not supported")


def get_screen_size(os: SupportedOS):
    if os == SupportedOS.LINUX:
        return get_screen_size_linux()
    if os == SupportedOS.WINDOWS:
        return get_screen_size_windows()
    else:
        raise NotImplementedError(f"{os=} not supported")



from abc import ABC, abstractmethod
from typing import *

from cvvideoplayer.frame_editors import BaseFrameEditor
from cvvideoplayer.utils.bbox_utils import Bbox
from cvvideoplayer.utils.drawing_utils import draw_rectangle, draw_label
from cvvideoplayer.utils.video_player_utils import KeyFunction


class BaseBboxPlotter(BaseFrameEditor, ABC):
    """
    An abstract class that can be used a parent to any FrameEditor that needs to print bounding boxes on the frame.
    The self._edit_frame is already implemented and instead the derived class must implement the "get_bboxes" method.
    """

    def __init__(
        self,
        enable_by_default: bool = False,
        show_above_bbox_label: bool = True,
        show_below_bbox_label: bool = True,
        default_bbox_color: Tuple[int, int, int] = (255, 0, 0),
        text_color: Tuple[int, int, int] = (255, 255, 255),
        drawing_thickness: int = 1,
        font_scale: float = 0.5,
        label_text_color: Tuple[int, int, int] = (255, 255, 255),
        label_filling_color: Tuple[int, int, int] = (0, 0, 0),
    ):
        super().__init__(enable_by_default)
        self._text_color = text_color
        self._font_scale = font_scale
        self._thickness = drawing_thickness
        self._show_above_bbox_label = show_above_bbox_label
        self._show_below_bbox_label = show_below_bbox_label
        self._default_bbox_color = default_bbox_color
        self._label_text_color = label_text_color
        self._label_filling_color = label_filling_color

    @abstractmethod
    def get_bboxes(self, frame, frame_num) -> List[Bbox]:
        pass

    @property
    def edit_after_resize(self) -> bool:
        return False

    @property
    def key_function_to_register(self):
        return [
            KeyFunction(key="d", func=self.enable_disable, description="Show/Hide Detections"),
            KeyFunction(key="l", func=self.toggle_show_above_bbox_label, description="Show/Hide above label"),
            KeyFunction(key="b", func=self.toggle_show_below_bbox_label, description="Show/Hide below label"),
            KeyFunction(key="ctrl+i", func=lambda: self.change_font_size(0.1), description="increase label size"),
            KeyFunction(key="ctrl+u", func=lambda: self.change_font_size(-0.1), description="decrease label size"),
        ]

    def _edit_frame(self, frame, frame_num):
        for bbox in self.get_bboxes(frame, frame_num):
            draw_rectangle(
                frame,
                x=bbox.x1,
                y=bbox.y1,
                w=bbox.width,
                h=bbox.height,
                color=bbox.color or self._default_bbox_color,
                thickness=self._thickness,
            )
            if self._show_above_bbox_label and bbox.above_label is not None:
                draw_label(
                    frame=frame,
                    bbox_x1=bbox.x1,
                    bbox_y1=bbox.y1,
                    bbox_h=bbox.height,
                    below_or_above="above",
                    text=bbox.above_label,
                    font_scale=self._font_scale,
                    thickness=self._thickness,
                    label_line_color=bbox.color or self._default_bbox_color,
                    text_color=self._label_text_color,
                    filling_color=self._label_filling_color,
                )
            if self._show_below_bbox_label and bbox.below_label is not None:
                draw_label(
                    frame=frame,
                    bbox_x1=bbox.x1,
                    bbox_y1=bbox.y1,
                    bbox_h=bbox.height,
                    below_or_above="below",
                    text=bbox.below_label,
                    font_scale=self._font_scale,
                    thickness=self._thickness,
                    label_line_color=bbox.color or self._default_bbox_color,
                    text_color=self._label_text_color,
                    filling_color=self._label_filling_color,
                )
        return frame

    def toggle_show_above_bbox_label(self):
        self._show_above_bbox_label = not self._show_above_bbox_label

    def toggle_show_below_bbox_label(self):
        self._show_below_bbox_label = not self._show_below_bbox_label

    def change_font_size(self, by: float):
        self._font_scale = max(0.1, min(5.0, self._font_scale + by))



from abc import ABC, abstractmethod
from typing import Optional, List

import numpy as np

from ..utils.video_player_utils import KeyFunction


class BaseFrameEditor(ABC):

    def __init__(self, enable_by_default):
        self._enabled = enable_by_default

    @property
    @abstractmethod
    def edit_after_resize(self) -> bool:
        """
        Returns a boolean indicating whether the edit should happen before the frame is resized to fit the frame or
        after. True for after...
        """
        pass

    @abstractmethod
    def _edit_frame(self, frame: np.ndarray, frame_num: int) -> np.ndarray:
        """
        Here is where the editing happens. The function receives a frame and frame number and should return the frame
        after it has been altered in any way desirable by the user

        Args:
            frame (): the input frame
            frame_num ():

        Returns: the edited frame
        """
        pass

    def enable_disable(self):
        self._enabled = not self._enabled

    @property
    def key_function_to_register(self) -> Optional[List[KeyFunction]]:
        """
        Optionally return a list of KeyFunctions to be registered once the frame editor is added to the video player
        Examples:
            1. [
                  KeyFunction(
                      key="ctrl+f",  # A standard key combination can be any combination of modifiers with a single key
                      func=self.enable_disable,  # A function that receives no input
                      description="disable enable"  # A description of whatever this frame editor does
                  ),
                  KeyFunction(
                      key="ctrl+num", # A general num function that handles any number press
                      func=self._change_id, # this function needs to receive the number as an input
                      description="change the printed id according to the number pressed"
                  )
              ]
        """
        return None

    def edit_frame(self, frame: np.ndarray, frame_num: int) -> np.ndarray:
        if not self._enabled:
            return frame
        else:
            return self._edit_frame(frame, frame_num)

    def setup(self, frame) -> None:
        """
        Optionally configure more parameters according to the first incoming frame
        """
        return

    def teardown(self) -> None:
        """
        Optionally define how the editor should close when video player is closed
        """
        return



import numpy as np

from ..base_frame_editor import BaseFrameEditor
from ...utils.video_player_utils import hist_eq_uint16, KeyFunction


class HistogramEqualizer(BaseFrameEditor):
    def __init__(self, enable_by_default: bool = False):
        super().__init__(enable_by_default)

    @property
    def key_function_to_register(self):
        return [
            KeyFunction(
                key="ctrl+alt+h",
                func=self.enable_disable,
                description="Enable/Disable histogram equalization",
            ),
        ]

    def _edit_frame(self, frame: np.ndarray, frame_num: int) -> np.ndarray:
        if frame.dtype == "uint8":
            norm_factor = 255
        elif frame.dtype == "uint16":
            norm_factor = 65535
        else:
            raise ValueError(f"image must be either Uint8 or Uint16 but got {frame.dtype}")

        frame = frame.astype("float")
        frame /= norm_factor

        frame = (frame * (2**16 - 1)).astype("uint16")
        frame = (hist_eq_uint16(frame) / 255).astype("uint8")
        return frame

    @property
    def edit_after_resize(self) -> bool:
        return False



from .frame_reader import *
from .recorder import Recorder
from .video_player import VideoPlayer
from .utils.bbox_utils import Bbox
from .utils.video_player_utils import KeyFunction
from .input_manager import InputManager



from typing import Dict, Tuple

import numpy as np

from cvvideoplayer.utils.video_player_utils import KeyFunction, write_text_on_img
from ..base_frame_editor import BaseFrameEditor
from ...input_manager import InputManager


class KeyMapOverlay(BaseFrameEditor):
    def __init__(
        self,
        enable_by_default: bool = True,
        video_total_frame_num=None,
        font_scale: float = 1,
        font_thickness: int = 1,
        top_left_coordinate: Tuple[int, int] = (70, 10),
    ):
        super().__init__(enable_by_default)
        self._video_total_frame_num = video_total_frame_num
        self._font_scale = font_scale
        self._font_thickness = font_thickness
        self._tl_coordinate = top_left_coordinate
        self._key_map: Dict[str:KeyFunction] = {}

    @property
    def key_function_to_register(self):
        return [
            KeyFunction(key="ctrl+k", func=self.enable_disable, description="Show/Hide key map"),
        ]

    def setup(self, _) -> None:
        self._key_map = InputManager().clone_keymap()

    def _edit_frame(self, frame: np.ndarray, frame_num: int) -> np.ndarray:
        row = self._tl_coordinate[0]
        for key, key_function in self._key_map.items():
            if not key_function.description:
                continue
            write_text_on_img(
                frame,
                f"{key:20.20}{key_function.description:60.60}",
                row=row,
                col=self._tl_coordinate[1],
                font_scale=self._font_scale,
                thickness=self._font_thickness,
            )
            row += int(20 * self._font_scale)

        return frame

    @property
    def edit_after_resize(self) -> bool:
        return True



import platform
import time
from typing import Optional, List
from functools import partial

import cv2
import numpy as np

from .input_manager import InputManager, SupportedOS
from .frame_editors import FrameInfoOverlay, FrameNormalizer, HistogramEqualizer, BaseFrameEditor, KeyMapOverlay
from .frame_reader import FrameReader
from .recorder import Recorder
from .utils.video_player_utils import (
    get_screen_adjusted_frame_size,
    KeyFunction,
    is_window_closed_by_mouse_click,
    get_screen_size,
    get_in_focus_window_id,
)


class VideoPlayer:
    def __init__(
        self,
        video_name: str,
        frame_reader: FrameReader,
        start_from_frame: int = 0,
        recorder: Optional[Recorder] = None,
        add_basic_frame_editors: bool = True,
    ):
        self._video_name = video_name
        self._frame_reader = frame_reader
        self._recorder = recorder

        self._last_frame = len(frame_reader) - 1
        self._frame_num = start_from_frame - 1
        self._current_system = SupportedOS(platform.system())
        self._screen_size = get_screen_size(self._current_system)

        self._current_frame = None
        self._frame_editors: List[BaseFrameEditor] = []

        self._play = False

        self._resize_factor = 1.0
        self._play_speed = 1
        self._exit = False

        if add_basic_frame_editors:
            self._add_basic_frame_editors()

        self._add_default_key_functions()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._recorder is not None:
            self._recorder.teardown()
        for frame_editor in self._frame_editors:
            frame_editor.teardown()

    def add_frame_editor(self, frame_editor: BaseFrameEditor) -> None:
        assert isinstance(frame_editor, BaseFrameEditor), "frame_editor must be a derived class of BaseFrameEditor"
        self._frame_editors.append(frame_editor)
        if frame_editor.key_function_to_register is not None:
            for key_function in frame_editor.key_function_to_register:
                assert isinstance(key_function, KeyFunction), (
                    f"{frame_editor.__class__.__name__} is trying to register a keymap action"
                    f" (key = {key_function.key}) which is not an instance of KeymapAction"
                )
                InputManager().register_key_function(key_function)

    def run(self) -> None:
        self._setup()

        InputManager().start()

        while not self._exit:

            key = InputManager().get_input()

            if is_window_closed_by_mouse_click(window_name=self._video_name):
                cv2.destroyAllWindows()
                break
            if key == "mouse_click":
                cv2.waitKey(30)
            elif get_in_focus_window_id(self._current_system) != self._window_id:
                continue
            else:
                InputManager().handle_key_str(key)
                self._show_current_frame()

        cv2.destroyAllWindows()

    def _setup(self) -> None:
        self._next_frame(1)
        for frame_editor in self._frame_editors:
            frame_editor.setup(self._current_frame)

        self._show_current_frame()
        time.sleep(0.5)  # make sure video player is up before checking the window id
        self._window_id = get_in_focus_window_id(self._current_system)
        cv2.waitKey(50)

    def _show_current_frame(self) -> None:
        frame = self._current_frame.copy()

        for frame_editor in self._frame_editors:
            if not frame_editor.edit_after_resize:
                shape_before_edit = frame.shape[:2]
                frame = frame_editor.edit_frame(frame, self._frame_num)
                assert (
                    frame.shape[:2] == shape_before_edit
                ), "frame editors with edit_after_resize==False can not alter the frame's shape"

        frame = self._resize_frame(frame)

        for frame_editor in self._frame_editors:
            if frame_editor.edit_after_resize:
                frame = frame_editor.edit_frame(frame, self._frame_num)

        frame = self._resize_frame(frame)

        if self._recorder is not None:
            self._recorder.write_frame_to_video(frame)

        cv2.imshow(winname=self._video_name, mat=frame)
        cv2.waitKey(10)
        cv2.waitKey(1)  # for some reason windows OS requires an additional waitKey to work properly

    def _add_basic_frame_editors(self) -> None:
        self.add_frame_editor(FrameInfoOverlay(video_total_frame_num=len(self._frame_reader)))
        self.add_frame_editor(FrameNormalizer())
        self.add_frame_editor(HistogramEqualizer())
        self.add_frame_editor(KeyMapOverlay())

    def _play_continuously(self) -> None:
        while (not InputManager().has_input()) and self._play:
            if self._play_speed > 0:
                self._next_frame(self._play_speed)
            else:
                self._prev_frame(-self._play_speed)
            self._show_current_frame()

    def _resize_frame(self, frame) -> np.ndarray:
        width, height = get_screen_adjusted_frame_size(
            screen_size=self._screen_size,
            frame_width=frame.shape[1],
            frame_height=frame.shape[0],
        )
        frame_size = int(self._resize_factor * width), int(self._resize_factor * height)
        return cv2.resize(frame, frame_size)

    def _next_frame(self, num_frames_to_skip=1) -> None:
        if self._frame_num == self._last_frame:
            self._play = False
            return

        self._frame_num = min(self._frame_num + num_frames_to_skip, self._last_frame)
        self._current_frame = self._frame_reader.get_frame(self._frame_num)

    def _prev_frame(self, num_frames_to_skip=1) -> None:
        if self._frame_num == 0:
            return

        self._frame_num = max(self._frame_num - num_frames_to_skip, 0)
        self._current_frame = self._frame_reader.get_frame(self._frame_num)

    def _change_frame_resize_factor(self, change_by: float) -> None:
        self._resize_factor = max(0.1, min(1.0, self._resize_factor + change_by))

    def _play_pause(self):
        self._play = not self._play
        self._play_continuously()

    def _increase_play_speed(self) -> None:
        if self._play_speed <= -1:
            self._play_speed = 1
        else:
            self._play_speed = min(16, self._play_speed * 2)

        self._play = True
        self._play_continuously()

    def _decrease_play_speed(self) -> None:
        if self._play_speed >= 1:
            self._play_speed = -1
        else:
            self._play_speed = max(-16, self._play_speed * 2)

        self._play = True
        self._play_continuously()

    def _on_exit(self) -> None:
        self._exit = True

    def _add_default_key_functions(self) -> None:
        default_key_functions = [
            KeyFunction("space", self._play_pause, "Play/Pause video"),
            KeyFunction("right", partial(self._next_frame, 1), "Next frame"),
            KeyFunction("left", partial(self._prev_frame, 1), "Previous frame"),
            KeyFunction("ctrl+right", partial(self._next_frame, 10), "10 frames forward"),
            KeyFunction("ctrl+left", partial(self._prev_frame, 10), "10 frames back"),
            KeyFunction("ctrl+shift+right", partial(self._next_frame, 50), "50 frames forward"),
            KeyFunction("ctrl+shift+left", partial(self._prev_frame, 50), "50 frames back"),
            KeyFunction("ctrl++", partial(self._change_frame_resize_factor, 0.1), "Increase frame size"),
            KeyFunction("ctrl+=", partial(self._change_frame_resize_factor, 0.1), ""),
            KeyFunction("ctrl+-", partial(self._change_frame_resize_factor, -0.1), "Decrease frame size"),
            KeyFunction("+", self._increase_play_speed, "Increase play speed"),
            KeyFunction("shift++", self._increase_play_speed, ""),
            KeyFunction("=", self._increase_play_speed, ""),
            KeyFunction("-", self._decrease_play_speed, "Decrease play speed"),
            KeyFunction("esc", self._on_exit, "Exit gracefully"),
        ]

        for key_function in default_key_functions:
            InputManager().register_key_function(key_function)



from .base_frame_editor import BaseFrameEditor
from .base_bbox_plotter import BaseBboxPlotter
from .basic_frame_editors.frame_normlizer import FrameNormalizer
from .basic_frame_editors.frame_info_overlay import FrameInfoOverlay
from .basic_frame_editors.histogram_equalizer import HistogramEqualizer
from .basic_frame_editors.key_map_overlay import KeyMapOverlay



"""
This module contains an drawing routines based on OpenCV.
"""

from typing import Tuple, Optional

import cv2
import numpy as np


def draw_rectangle(
    image,
    x,
    y,
    w,
    h,
    color,
    thickness=2,
    only_corners=True,
):
    """Draw a rectangle."""
    tl = int(x), int(y)
    br = int(x + w), int(y + h)

    if not only_corners:
        cv2.rectangle(image, tl, br, color, thickness, lineType=cv2.LINE_AA)
        return

    tr = int(x + w), int(y)
    bl = int(x), int(y + h)

    cv2.line(image, tl, (tl[0] + w // 4, tl[1]), color, thickness, lineType=cv2.LINE_AA)
    cv2.line(image, tl, (tl[0], tl[1] + h // 4), color, thickness, lineType=cv2.LINE_AA)
    cv2.line(image, tr, (tr[0] - w // 4, tr[1]), color, thickness, lineType=cv2.LINE_AA)
    cv2.line(image, tr, (tr[0], tr[1] + h // 4), color, thickness, lineType=cv2.LINE_AA)
    cv2.line(image, bl, (bl[0] + w // 4, bl[1]), color, thickness, lineType=cv2.LINE_AA)
    cv2.line(image, bl, (bl[0], bl[1] - h // 4), color, thickness, lineType=cv2.LINE_AA)
    cv2.line(image, br, (br[0] - w // 4, br[1]), color, thickness, lineType=cv2.LINE_AA)
    cv2.line(image, br, (br[0], br[1] - h // 4), color, thickness, lineType=cv2.LINE_AA)


def draw_polygon(
    image,
    points,
    color,
    thickness=2,
):
    """Draw a rectangle."""
    points = np.array(points).reshape((-1, 1, 2)).astype(np.int32)
    cv2.polylines(image, [points], isClosed=True, color=color, thickness=thickness, lineType=cv2.LINE_AA)


def draw_label(
    frame: np.ndarray,
    bbox_x1: int,
    bbox_y1: int,
    bbox_h: int,
    below_or_above: str,
    text: str,
    font_scale: float,
    thickness: int,
    text_color: Tuple[int, int, int],
    label_line_color: Optional[Tuple[int, int, int]] = None,
    filling_color: Optional[Tuple[int, int, int]] = None,
):
    assert below_or_above in {"below", "above"}

    text_size = cv2.getTextSize(text, cv2.FONT_HERSHEY_PLAIN, fontScale=font_scale, thickness=thickness)

    if below_or_above == "above":
        label_pt1 = int(bbox_x1), int(bbox_y1 - 10 - text_size[0][1])
        label_pt2 = int(bbox_x1 + 10 + text_size[0][0]), int(bbox_y1)
        text_position = label_pt1[0] + 5, label_pt2[1] - 5
    else:
        label_pt1 = int(bbox_x1), int(bbox_y1 + bbox_h)
        label_pt2 = int(bbox_x1 + 10 + text_size[0][0]), int(bbox_y1 + bbox_h + 10 + text_size[0][1])
        text_position = label_pt1[0] + 5, label_pt2[1] - 5

    if filling_color is not None:
        cv2.rectangle(frame, label_pt1, label_pt2, color=filling_color, thickness=-1, lineType=cv2.LINE_AA)
    if label_line_color is not None:
        cv2.rectangle(frame, label_pt1, label_pt2, color=label_line_color, thickness=thickness, lineType=cv2.LINE_AA)

    cv2.putText(
        img=frame,
        text=text,
        org=text_position,
        fontFace=cv2.FONT_HERSHEY_PLAIN,
        fontScale=font_scale,
        color=text_color,
        thickness=thickness,
        lineType=cv2.LINE_AA,
    )



from typing import Union, Optional, Dict

import matplotlib.pyplot as plt
import numpy as np

from ..base_frame_editor import BaseFrameEditor
from ...utils.video_player_utils import KeyFunction


class FrameNormalizer(BaseFrameEditor):
    def __init__(
        self,
        enable_by_default: bool = True,
        range_min: Union[str, int] = "",
        range_max: Union[str, int] = "",
    ):
        super().__init__(enable_by_default)
        self._range_min = range_min
        self._range_max = range_max
        self._last_frame = None

    def setup(self, frame) -> None:
        self._last_frame = self._edit_frame(frame, frame_num=0)

    def set_dynamic_range(self):
        self._range_min = input("Set new image min: ")
        self._range_max = input("Set new image max: ")

    @property
    def key_function_to_register(self):
        return [
            KeyFunction(key="ctrl+r", func=self.set_dynamic_range, description="Set dynamic range"),
            KeyFunction(key="ctrl+alt+r", func=self.show_frame_histogram, description="Show frame histogram"),
        ]

    def _edit_frame(self, frame: np.ndarray, frame_num: int) -> np.ndarray:
        if frame.dtype == "uint8":
            if self._range_min == self._range_max == "":
                self._last_frame = frame
                return frame
            norm_factor = 255
        elif frame.dtype == "uint16":
            if self._range_min == self._range_max == "":
                self._last_frame = frame
                return (frame / 255).astype("uint8")
            norm_factor = 65535
        else:
            raise ValueError(f"image must be either Uint8 or Uint16 but got {frame.dtype}")

        frame = frame.astype("float")
        frame /= norm_factor

        norm_min = int(self._range_min) / norm_factor if self._range_min else 0
        norm_max = int(self._range_max) / norm_factor if self._range_max else norm_factor
        frame = (frame - norm_min) / (norm_max - norm_min)
        frame = np.clip(frame, 0, 1)
        frame = (frame * 255).astype("uint8")
        self._last_frame = frame
        return frame

    def show_frame_histogram(self):
        plt.hist(self._last_frame.ravel(), 256, (0, 256))
        plt.show()

    @property
    def edit_after_resize(self) -> bool:
        return False



from setuptools import setup, find_packages

setup(
    name="cvvideoplayer",
    version="0.1.4",
    author="Daniel Tomer",
    author_email="danieltomer1@gmail.com",
    description="moduler multi purpose video player for python",
    long_description=(
        "CV video player is a Python-based customizable video player that helps"
        " computer vision practitioners to develop, analyze and debug their video"
        " related algorithms and model."
    ),
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=[
        "numpy",
        "opencv-python",
        "pynput",
        "python-xlib",
        "matplotlib",
    ],
    python_requires=">=3.8",
    project_urls={
        "Homepage": "https://github.com/danieltomer1/CVVideoPlayer",
    },
)
